"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Corner = void 0;
const mapShortCellId_1 = require("./utils/mapShortCellId");
class Corner {
    sourceCell;
    directionFromSourceCell;
    neighborCells;
    id;
    /**
     * All cells, that are connected to this corner
     */
    adjacentCells;
    /**
     * @param sourceCell
     * @param directionFromSourceCell
     * @param neighborCells All cells, that are connected to the sourceCell through this corner
     */
    constructor(sourceCell, directionFromSourceCell, neighborCells) {
        this.sourceCell = sourceCell;
        this.directionFromSourceCell = directionFromSourceCell;
        this.neighborCells = neighborCells;
        this.adjacentCells = [this.sourceCell, ...this.neighborCells];
        const neighborCellIds = this.neighborCells
            .map((adjacentCell) => (0, mapShortCellId_1.mapShortCellId)(adjacentCell))
            .filter(Boolean);
        const cellIds = neighborCellIds.length > 0
            ? [(0, mapShortCellId_1.mapShortCellId)(sourceCell), ...neighborCellIds].toSorted().join('|')
            : [(0, mapShortCellId_1.mapShortCellId)(sourceCell), directionFromSourceCell].join('|');
        this.id = `corner[${cellIds}]`;
    }
}
exports.Corner = Corner;
