"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Edge = void 0;
const mapShortCellId_1 = require("./utils/mapShortCellId");
/**
 * The representation of an edge of a cell
 */
class Edge {
    edges;
    getPreviousEdgeDirection;
    getNextEdgeDirection;
    sourceCell;
    direction;
    neighborCell;
    id;
    /**
     * @param edges The edges instance
     * @param getPreviousEdgeDirection
     * @param getNextEdgeDirection
     * @param sourceCell
     * @param direction
     * @param neighborCell
     */
    constructor(edges, getPreviousEdgeDirection, getNextEdgeDirection, sourceCell, direction, neighborCell) {
        this.edges = edges;
        this.getPreviousEdgeDirection = getPreviousEdgeDirection;
        this.getNextEdgeDirection = getNextEdgeDirection;
        this.sourceCell = sourceCell;
        this.direction = direction;
        this.neighborCell = neighborCell;
        const cellIds = neighborCell
            ? [(0, mapShortCellId_1.mapShortCellId)(sourceCell), (0, mapShortCellId_1.mapShortCellId)(neighborCell)].toSorted().join('|')
            : [(0, mapShortCellId_1.mapShortCellId)(sourceCell), direction].join('|');
        this.id = `edge[${cellIds}]`;
    }
    /**
     * @returns The previous connected edge (e.g. for a square, the previous edge of "top" would be "left")
     * @protected
     */
    getPreviousEdge() {
        return this.edges.get(this.getPreviousEdgeDirection(this.direction));
    }
    /**
     * @returns The next connected edge (e.g. for a square, the next edge of "top" would be "right")
     * @protected
     */
    getNextEdge() {
        return this.edges.get(this.getNextEdgeDirection(this.direction));
    }
}
exports.Edge = Edge;
