import { GridEventDispatcher } from './utils';
/**
 * The column of a grid contains all information of the columns' cells
 */
export class Column {
    grid;
    col;
    id;
    _cells = [];
    eventDispatcher;
    /**
     * @param grid The grid this column is part of
     * @param col The column coordinate inside the grid
     */
    constructor(grid, col) {
        this.grid = grid;
        this.col = col;
        this.id = `column|${col}`;
        for (let row = 0; row < this.grid.height; row++) {
            this.cells.push(this.getCell(row));
        }
        this.eventDispatcher = new GridEventDispatcher();
        const forwardEvent = (event) => this.eventDispatcher.dispatchCellValueChangedEvent(event);
        for (const cell of this.cells) {
            cell.onValueChanged(forwardEvent);
        }
    }
    /**
     * @returns all cells of this column
     */
    get cells() {
        return this._cells;
    }
    /**
     * @param row The row coordinate of the grid
     * @returns A cell
     */
    getCell(row) {
        return this.grid.getCell({ col: this.col, row });
    }
    /**
     * @param callback A function that should be called, when a cell value of this column changes
     * @returns a function to unregister the callback
     */
    onCellValueChanged(callback) {
        return this.eventDispatcher.onCellValueChanged(callback);
    }
}
