import { mapShortCellId } from './utils/mapShortCellId';
/**
 * The representation of an edge of a cell
 */
export class Edge {
    edges;
    getPreviousEdgeDirection;
    getNextEdgeDirection;
    sourceCell;
    direction;
    neighborCell;
    id;
    /**
     * @param edges The edges instance
     * @param getPreviousEdgeDirection
     * @param getNextEdgeDirection
     * @param sourceCell
     * @param direction
     * @param neighborCell
     */
    constructor(edges, getPreviousEdgeDirection, getNextEdgeDirection, sourceCell, direction, neighborCell) {
        this.edges = edges;
        this.getPreviousEdgeDirection = getPreviousEdgeDirection;
        this.getNextEdgeDirection = getNextEdgeDirection;
        this.sourceCell = sourceCell;
        this.direction = direction;
        this.neighborCell = neighborCell;
        const cellIds = neighborCell
            ? [mapShortCellId(sourceCell), mapShortCellId(neighborCell)].toSorted().join('|')
            : [mapShortCellId(sourceCell), direction].join('|');
        this.id = `edge[${cellIds}]`;
    }
    /**
     * @returns The previous connected edge (e.g. for a square, the previous edge of "top" would be "left")
     * @protected
     */
    getPreviousEdge() {
        return this.edges.get(this.getPreviousEdgeDirection(this.direction));
    }
    /**
     * @returns The next connected edge (e.g. for a square, the next edge of "top" would be "right")
     * @protected
     */
    getNextEdge() {
        return this.edges.get(this.getNextEdgeDirection(this.direction));
    }
}
