import type { Coordinate } from './Coordinate';
import { type CellValueChangedEvent } from './utils';
import type { Row } from './Row';
import type { Column } from './Column';
import type { Neighbors } from './Neighbors';
import type { Edges } from './Edges';
import type { Corners } from './Corners';
import type { Directions } from './Directions';
/**
 * A Cell is part of a grid. It contains meta information like its coordinates inside the grid and the corresponding value.
 */
export declare abstract class Cell<TValue, TDirections extends Directions> implements Coordinate {
    readonly id: string;
    readonly row: number;
    readonly col: number;
    abstract readonly neighbors: Neighbors<TValue, TDirections, Cell<TValue, TDirections>>;
    abstract readonly edges: Edges<TValue, TDirections, Cell<TValue, TDirections>>;
    abstract readonly corners: Corners<TValue, TDirections, Cell<TValue, TDirections>>;
    private _value;
    private readonly eventDispatcher;
    /**
     * @param coordinate The coordinate in the grid
     * @param value The value of the cell
     */
    protected constructor(coordinate: Coordinate, value: TValue);
    /**
     * @returns the value of the cell
     */
    get value(): TValue;
    /**
     * Changes the value of the cell
     */
    set value(value: TValue);
    /**
     * @param callback A function that should be called, when the cell value changes
     * @returns a function to unregister the callback
     */
    onValueChanged(callback: (event: CellValueChangedEvent<TValue, TDirections>) => void): () => void;
    /**
     * @returns The row of the cell
     */
    abstract getRow(): Row<TValue, TDirections, Cell<TValue, TDirections>>;
    /**
     @returns The column of the cell
     */
    abstract getColumn(): Column<TValue, TDirections, Cell<TValue, TDirections>>;
    /**
     * @param cloneValue A custom function to clone the value of this cell (defaults to copying the value)
     * @returns The cloned cell
     */
    abstract clone(cloneValue?: (value: TValue) => TValue): Cell<TValue, TDirections>;
}
//# sourceMappingURL=Cell.d.ts.map